package com.dsic.gj_erp.util;

import org.apache.commons.lang3.StringUtils;

import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * 对象转换工具类
 *
 * @Author xhj
 * @date 2021/1/18
 */
@SuppressWarnings("ALL")
public class ObjConvertUtils {

    /**
     * 获取类的所有属性，包括父类
     *
     * @param object
     * @return
     */
    public static Field[] getAllFields(Object object) {
        Class<?> clazz = object.getClass();
        List<Field> fieldList = new ArrayList<>();
        while (clazz != null) {
            fieldList.addAll(new ArrayList<>(Arrays.asList(clazz.getDeclaredFields())));
            clazz = clazz.getSuperclass();
        }
        Field[] fields = new Field[fieldList.size()];
        fieldList.toArray(fields);
        return fields;
    }

    public static boolean isEmpty(Object object) {
        if (object == null) {
            return (true);
        }
        if ("".equals(object)) {
            return (true);
        }
        if ("null".equals(object)) {
            return (true);
        }
        return (false);
    }

    public static boolean checkObjAllFieldsIsNull(Object object) {
        if (null == object) {
            return true;
        }

        try {
            for (Field f : object.getClass().getDeclaredFields()) {
                f.setAccessible(true);

                System.out.print(f.getName() + ":");
                System.out.println(f.get(object));

                if (f.get(object) != null && StringUtils.isNotBlank(f.get(object).toString())) {
                    return false;
                }

            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        return true;
    }

    public static Object checkNullNew(Object obj) {
        Class<? extends Object> clazz = obj.getClass();
        // 获取实体类的所有属性，返回Field数组
        Field[] fields = clazz.getDeclaredFields();
        for (Field field : fields) {
            // 可访问私有变量
            field.setAccessible(true);
            // 获取属性类型
            String type = field.getGenericType().toString();

            try {
                if ("class java.lang.String".equals(type)) {
                    handleStringField(obj, field);
                } else if ("class java.lang.Integer".equals(type)) {
                    handleIntegerField(obj, field);
                } else if ("class java.lang.Double".equals(type)) {
                    handleDoubleField(obj, field);
                } else if ("class java.math.BigDecimal".equals(type)) {
                    handleBigDecimalField(obj, field);
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return obj;
    }

    // 处理String类型
    private static void handleStringField(Object obj, Field field) throws Exception {
        String methodName = field.getName().replaceFirst(field.getName().substring(0, 1),
            field.getName().substring(0, 1).toUpperCase());
        Method methodGet = obj.getClass().getMethod("get" + methodName);
        String str = (String) methodGet.invoke(obj);
        if (StringUtils.isBlank(str)) {
            field.set(obj, "");
        }
    }

    // 处理Integer类型
    private static void handleIntegerField(Object obj, Field field) throws Exception {
        Integer value = (Integer) field.get(obj);
        if (value == null) {
            field.set(obj, 0);
        }
    }

    // 处理Double类型
    private static void handleDoubleField(Object obj, Field field) throws Exception {
        Double value = (Double) field.get(obj);
        if (value == null) {
            field.set(obj, 0.0);
        }
    }

    // 处理BigDecimal类型
    private static void handleBigDecimalField(Object obj, Field field) throws Exception {
        BigDecimal value = (BigDecimal) field.get(obj);
        if (value == null) {
            field.set(obj, BigDecimal.ZERO);
        }
    }
}